@@ -1,19 +1,16 @@
 package AllTests;
 
-use Test::Unit::TestSuite;
+use base 'Test::Unit::TestSuite';
 
 use File::Find ();
+use File::Basename ();
 use File::Spec ();
 
-sub new {
-    return bless {}, shift;
-}
-
 sub suite {
     my $class = shift;
     my $suite = Test::Unit::TestSuite->empty_new("Framework Tests");
 
-    my $dir = (File::Spec->splitpath(__FILE__))[1]; $dir =~ s{/$}{};
+    my $dir = (File::Basename::dirname(__FILE__));
     my $depth = scalar File::Spec->splitdir($dir);
 
     File::Find::find({
@@ -25,6 +22,7 @@ sub suite {
             return unless scalar @path > 0;
             my $class = join '::', @path;
             return unless $class;
+            return if $class =~ /^Test::Unit::/;
             return if @ARGV and $class !~ $ARGV[0];
             return unless eval "use $class; $class->isa('Test::Unit::TestCase');";
             $suite->add_test($class);
